/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :
* Device(s)    : CMS32L051
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2022/5/30
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "uart_demo.h"
#include "dma_demo.h"
#include "spihs_demo.h"
#include "gpio_demo.h"
#include "delay_demo.h"
#include "drv_w25qxx.h"
#define SPI_WITH_DMA
#define W25QXX_DEVICE
/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
uint8_t mrx0_buf[13];
uint8_t mtx_buf[] = {0xA5, 0x5A, 0xCC, 0x33, 0x3C, 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
extern uint8_t g_spi_tx_end, g_spi_rx_end;


int main()
{
    uint32_t msCnt;     // count value of 1ms
    uint32_t i, err;
    uint32_t tmp;
    uint8_t uniq_id[8];
    //-----------------------------------------------------------------------
    // Systick setting
    //-----------------------------------------------------------------------
    SystemCoreClockUpdate();
    msCnt = SystemCoreClock / 1000;
    SysTick_Config(msCnt);
    delay_init(SystemCoreClock);
    Led_Init();


    Uart0_Init(19200);
    Spihs_Init();

#ifdef  W25QXX_DEVICE
    tmp = drv_w25qxx_read_device_id();

    printf("W25QXX Device ID: %02X.\n", tmp);

    tmp = drv_w25qxx_read_jedec_id();

    printf("W25QXX JEDEC ID: %03X.\n", tmp);

    drv_w25qxx_read_uniq_id(uniq_id);

    printf("W25QXX UNIQ ID: ");

    for (int i = 0; i < sizeof(uniq_id); i++)
    {
        printf("%02X ", uniq_id[i]);
    }

    printf(".\n");
#else
#ifdef SPI_WITH_DMA
    Spihs_Dma_Write(mtx_buf, sizeof(mtx_buf));

    while (g_spi_tx_end == 0);

#else
    Spihs_Int_Write(mtx_buf, sizeof(mtx_buf));

    while (g_spi_tx_end == 0);

#endif

    delayMS(4);

#ifdef SPI_WITH_DMA
    Spihs_Dma_Read(mrx0_buf, sizeof(mrx0_buf));

    while (g_spi_rx_end == 0);

#else
    Spihs_Int_Read(mrx0_buf, sizeof(mrx0_buf));

    while (g_spi_rx_end == 0);

#endif
    delayMS(2);
    err = 0;

    for (i = 0; i < sizeof(mrx0_buf); i++)
    {
        if (mtx_buf[i] != mrx0_buf[i])
        {
            err++;
        }

        printf(" %02x", mrx0_buf[i]);
    }

    if (err)
    {
        for (i = 0; i < 8; i++)
        {
            LED_Toggle;
            delayMS(250);
        }

        LED_OFF;
    }
    else
    {
        for (i = 0; i < 8; i++)
        {
            LED_Toggle;
            delayMS(250);
        }

        LED_ON;
    }

#endif

}

