#include "motor.h"
#include <stdint.h>
/* Motor Driver */

volatile uint8_t  g_u1_v_pattern;             /* voltage pattern */
volatile uint8_t  g_u1_direction = 0;             /* rotation direction (0:CW ,1:CCW) */
volatile uint8_t  g_u1_v_pattern_open[2][7] = {
                           {0,
                            MTR_PATTERN_CW_U_V,         /* 5 */
                            MTR_PATTERN_CW_W_V,         /* 1 */
                            MTR_PATTERN_CW_W_U,         /* 3 */
                            MTR_PATTERN_CW_V_U,         /* 2 */
                            MTR_PATTERN_CW_V_W,         /* 6 */
                            MTR_PATTERN_CW_U_W},        /* 4 */
                           {0,
                            MTR_PATTERN_CCW_U_W,        /* 4 */
                            MTR_PATTERN_CCW_V_W,        /* 6 */
                            MTR_PATTERN_CCW_V_U,        /* 2 */
                            MTR_PATTERN_CCW_W_U,        /* 3 */
                            MTR_PATTERN_CCW_W_V,        /* 1 */
                            MTR_PATTERN_CCW_U_V}        /* 5 */
                  };                          /* array of voltage pattern for openloop drive */


/******************************************************************************
* Function Name : mtr_pattern_up_chopping
* @brief  Set voltage pattern upper arm chopping PWM
* @param  u1_pattern : voltage pattern
* @return none
******************************************************************************/
void mtr_pattern_up_chopping(uint8_t u1_pattern)
{
    /*** set voltage pattern ***/
    if (MTR_CW == g_u1_direction)               /* check rotation direction */
    {
        switch (u1_pattern)
        {
            case MTR_PATTERN_CW_W_V:            /* from W phase to V phase */
                g_u1_v_pattern = MTR_W_PWM_VN_ON;
                //mtr_set_tau_window_signal(MTR_TAU_TIMER_WP);
            break;

            case MTR_PATTERN_CW_U_V:            /* from U phase to V phase */
                g_u1_v_pattern = MTR_U_PWM_VN_ON;
                //mtr_set_tau_window_signal(MTR_TAU_TIMER_UP);
            break;

            case MTR_PATTERN_CW_U_W:            /* from U phase to W phase */
                g_u1_v_pattern = MTR_U_PWM_WN_ON;
                //mtr_set_tau_window_signal(MTR_TAU_TIMER_UP);
            break;

            case MTR_PATTERN_CW_V_W:            /* from V phase to W phase */
                g_u1_v_pattern = MTR_V_PWM_WN_ON;
                //mtr_set_tau_window_signal(MTR_TAU_TIMER_VP);
            break;

            case MTR_PATTERN_CW_V_U:            /* from V phase to U phase */
                g_u1_v_pattern = MTR_V_PWM_UN_ON;
                //mtr_set_tau_window_signal(MTR_TAU_TIMER_VP);
            break;

            case MTR_PATTERN_CW_W_U:            /* from W phase to U phase */
                g_u1_v_pattern = MTR_W_PWM_UN_ON;
                //mtr_set_tau_window_signal(MTR_TAU_TIMER_WP);
            break;

            default:                            /* pseudo Hall signal pattern error */
                g_u1_v_pattern = MTR_PATTERN_ERROR;
            break;
        }
    }
    else if (MTR_CCW == g_u1_direction)         /* check rotation direction */
    {
        switch (u1_pattern)
        {
            case MTR_PATTERN_CCW_W_V:           /* from W phase to V phase */
                g_u1_v_pattern = MTR_W_PWM_VN_ON;
                //mtr_set_tau_window_signal(MTR_TAU_TIMER_WP);
            break;

            case MTR_PATTERN_CCW_W_U:           /* from W phase to U phase */
                g_u1_v_pattern = MTR_W_PWM_UN_ON;
                //mtr_set_tau_window_signal(MTR_TAU_TIMER_WP);
            break;

            case MTR_PATTERN_CCW_V_U:           /* from V phase to U phase */
                g_u1_v_pattern = MTR_V_PWM_UN_ON;
                //mtr_set_tau_window_signal(MTR_TAU_TIMER_VP);
            break;

            case MTR_PATTERN_CCW_V_W:           /* from V phase to W phase */
                g_u1_v_pattern = MTR_V_PWM_WN_ON;
                //mtr_set_tau_window_signal(MTR_TAU_TIMER_VP);
            break;

            case MTR_PATTERN_CCW_U_W:           /* from U phase to W phase */
                g_u1_v_pattern = MTR_U_PWM_WN_ON;
                //mtr_set_tau_window_signal(MTR_TAU_TIMER_UP);
            break;

            case MTR_PATTERN_CCW_U_V:           /* from U phase to V phase */
                g_u1_v_pattern = MTR_U_PWM_VN_ON;
                //mtr_set_tau_window_signal(MTR_TAU_TIMER_UP);
            break;

            default:                            /* pseudo Hall signal pattern error */
                g_u1_v_pattern = MTR_PATTERN_ERROR;
            break;
        }
    }
    else
    {
        /* Do Nothing */
    }
}


/******************************************************************************
* Function Name : mtr_change_pattern
* @brief  Change voltage pattern
* @param  u1_pattern - Voltage pattern
* @return none
******************************************************************************/
void mtr_change_pattern(uint8_t u1_pattern)
{
    /*========================*/
    /*     stop PWM timer     */
    /*========================*/
    //TMM->TMSTR = 0x0C;                          /* stop TM0 */

    /*======================*/
    /*     set register     */
    /*======================*/
    /* PWM output arm setting */
    switch (u1_pattern)
    {
        case MTR_V_PWM_WN_ON:
            TMM->TMOER1 = 0xAF;               /* Vp/Vn : PWM output enable */
            MTR_PORT_UP_CLR();                /* Up = OFF("Low") */
            MTR_PORT_UN_CLR();                /* Un = OFF("Low") */
            MTR_PORT_VP_CLR();                /* Vp = OFF("Low") */
            MTR_PORT_VN_CLR();                /* Vn = OFF("Low") */
            MTR_PORT_WP_CLR();                /* Wp = OFF("Low") */
            MTR_PORT_WN_SET();                /* Wn = ON ("High") */
        break;

        case MTR_W_PWM_VN_ON:
            TMM->TMOER1 = 0x5F;               /* Wp/Wn : PWM output enable */
            MTR_PORT_UP_CLR();                /* Up = OFF("Low") */
            MTR_PORT_UN_CLR();                /* Un = OFF("Low") */
            MTR_PORT_VP_CLR();                /* Vp = OFF("Low") */
            MTR_PORT_VN_SET();                /* Vn = ON ("High") */
            MTR_PORT_WP_CLR();                /* Wp = OFF("Low") */
            MTR_PORT_WN_CLR();                /* Wn = OFF("Low") */
        break;

        case MTR_W_PWM_UN_ON:
            TMM->TMOER1 = 0x5F;               /* Wp/Wn : PWM output enable */
            MTR_PORT_UP_CLR();                /* Up = OFF("Low") */
            MTR_PORT_UN_SET();                /* Un = ON ("High") */
            MTR_PORT_VP_CLR();                /* Vp = OFF("Low") */
            MTR_PORT_VN_CLR();                /* Vn = OFF("Low") */
            MTR_PORT_WP_CLR();                /* Wp = OFF("Low") */
            MTR_PORT_WN_CLR();                /* Wn = OFF("Low") */
        break;

        case MTR_U_PWM_WN_ON:
            TMM->TMOER1 = 0xF5;               /* Up/Un : PWM output enable */
            MTR_PORT_UP_CLR();                /* Up = OFF("Low") */
            MTR_PORT_UN_CLR();                /* Un = OFF("Low") */
            MTR_PORT_VP_CLR();                /* Vp = OFF("Low") */
            MTR_PORT_VN_CLR();                /* Vn = OFF("Low") */
            MTR_PORT_WP_CLR();                /* Wp = OFF("Low") */
            MTR_PORT_WN_SET();                /* Wn = ON ("High") */
        break;

        case MTR_U_PWM_VN_ON:
            TMM->TMOER1 = 0xF5;               /* Up/Un : PWM output enable */
            MTR_PORT_UP_CLR();                /* Up = OFF("Low") */
            MTR_PORT_UN_CLR();                /* Un = OFF("Low") */
            MTR_PORT_VP_CLR();                /* Vp = OFF("Low") */
            MTR_PORT_VN_SET();                /* Vn = ON ("High") */
            MTR_PORT_WP_CLR();                /* Wp = OFF("Low") */
            MTR_PORT_WN_CLR();                /* Wn = OFF("Low") */
        break;

        case MTR_V_PWM_UN_ON:
            TMM->TMOER1 = 0xAF;               /* Vp/Vn : PWM output enable */
            MTR_PORT_UP_CLR();                /* Up = OFF("Low") */
            MTR_PORT_UN_SET();                /* Un = ON ("High") */
            MTR_PORT_VP_CLR();                /* Vp = OFF("Low") */
            MTR_PORT_VN_CLR();                /* Vn = OFF("Low") */
            MTR_PORT_WP_CLR();                /* Wp = OFF("Low") */
            MTR_PORT_WN_CLR();                /* Wn = OFF("Low") */
        break;

        /* the default case is intentionally combined */
        default:
        break;
    }
}
/******************************************************************************
* Function Name : mtr_pattern_set
* @brief  Set voltage pattern
* @param  u1_pattern - voltage pattern value
* @return none
******************************************************************************/
void mtr_pattern_set(uint8_t u1_pattern)
{
   /*** set voltage pattern ***/
    mtr_pattern_up_chopping(u1_pattern);        /* Upper arm chopping pattern */

    mtr_change_pattern(g_u1_v_pattern);         /* change voltage pattern */
}
